//////////////////////////////////////////////////
/////Relief Mapping with support for 3 Lights/////
//////////////////////////////////////////////////
//////////ps 2.a Relief Mapping in one pass	 	//
//////////ps 2.0 Parallax Mapping in two passes //
//////////Fixed Function Bumpmapping			 	//
//////////////////////////////////////////////////
//////////////////////////////////////////////////
////20.12.2007////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////

float4x4 matWorldViewProj;	
float4x4 matWorld;
float4x4 matWorldView;
float4 vecLightPos[8]; //light position
float4 vecLightColor[8]; //light position
float4 vecViewPos;
float4 vecFog;
float4 vecAmbient;
float4 vecSkill41;	//GlossStrength and GlossIntensity

#define Height vecSkill41[3]
#define linear_search_steps 15
#define binary_search_steps 5

texture entSkin1; //this is the color map
texture entSkin2; //this is the normal map
texture entSkin3; //this is the light map
texture entSkin4; //this is the height map

sampler ColorMapSampler = sampler_state
{
   Texture = <entSkin1>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = wrap;
   AddressV  = wrap;
};

sampler BumpMapSampler = sampler_state
{
   Texture = <entSkin2>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = wrap;
   AddressV  = wrap;
};

sampler GlossMapSampler = sampler_state
{
   Texture = <entSkin3>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = wrap;
   AddressV  = wrap;
};

sampler HeightMapSampler = sampler_state
{
   Texture = <entSkin4>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = wrap;
   AddressV  = wrap;
};


struct VS_OUT
{
   float4 Pos  	: POSITION;
    
   float2 Tex		: TEXCOORD0;
   float3 View 	: TEXCOORD1;
   
   float4 Light1	: TEXCOORD2;
   float4 Light2	: TEXCOORD3;
   float4 Light3	: TEXCOORD4;
   
   float4 vPos		: TEXCOORD5;
	float4 Tangent	: TEXCOORD6;
	float4 Binormal: TEXCOORD7;

   float Fog : FOG;
};


//////////////////////////////////////////////
////////////3 lights in one pass//////////////
//////////////////ps 2.a//////////////////////
//////////////////////////////////////////////

VS_OUT ps2a_VS(float4 Pos : POSITION, float2 texcoord0 : TEXCOORD0, float3 Normal : NORMAL, float4 Tangent : TEXCOORD2  )
{
    VS_OUT Out = (VS_OUT)0;
    Out.Pos = mul(Pos, matWorldViewProj);	// transform Position
    
    float3 Binormal = -cross(Normal,Tangent.xyz)*Tangent.w;

	// isolate matWorldView rotation-only part
	float3x3 modelViewRotXf;
	modelViewRotXf[0] = matWorldView[0].xyz;
	modelViewRotXf[1] = matWorldView[1].xyz;
	modelViewRotXf[2] = matWorldView[2].xyz;

	// vertex position in view space (with model transformations)
	Out.vPos.xyz = mul(Pos,matWorldView).xyz;

	// tangent space vectors in view space (with model transformations)
	Out.Tangent.xyz = mul(Tangent.xyz,modelViewRotXf);
	Out.Binormal.xyz = mul(Binormal,modelViewRotXf);
	float3 Temp_Normal = mul(Normal,modelViewRotXf);
	
	Out.vPos.w = Temp_Normal.x;
	Out.Tangent.w = Temp_Normal.y;
	Out.Binormal.w = Temp_Normal.z;

    // compute the 3x3 tranform matrix 
    // to transform from world space to tangent space
    float3x3 worldToTangentSpace;
    worldToTangentSpace[0] = mul(Tangent.xyz, matWorld);
    worldToTangentSpace[1] = mul(Binormal, matWorld);
    worldToTangentSpace[2] = mul(Normal, matWorld);

    Out.Tex = texcoord0;

    float3 PosWorld = mul(Pos, matWorld);

    float3 Viewer = PosWorld - vecViewPos;
    Out.View = mul(worldToTangentSpace, -Viewer);		// V

    //light 1
    float3 Light1 = PosWorld - vecLightPos[0]; 
    Out.Light1.xyz = mul(worldToTangentSpace, -Light1);	// L
    Out.Light1.w = distance(PosWorld,vecLightPos[0])/vecLightPos[0].w;
    
    //light 2
    float3 Light2 = PosWorld - vecLightPos[1];
    Out.Light2.xyz = mul(worldToTangentSpace, -Light2);	// L
    Out.Light2.w = distance(PosWorld,vecLightPos[1])/vecLightPos[1].w;
    
    //light 3
    float3 Light3 = PosWorld - vecLightPos[2];
    Out.Light3.xyz = mul(worldToTangentSpace, -Light3);	// L
    Out.Light3.w = distance(PosWorld,vecLightPos[2])/vecLightPos[2].w;

    float ofog = 1 - (distance(PosWorld, vecViewPos) - vecFog.x) * (vecFog.z);
    Out.Fog = ofog;	

   return Out;
}

//calculates the relief mapping texcoords
float2 ReliefMapping(sampler2D reliefmap,float2 TexCoord,float3 vPos,float3 Tangent,float3 Binormal,float3 Normal)
{
	//ray intersect in view direction
	float3 Vn = normalize(vPos);
	float a = dot(Normal,-Vn);
	float3 s  = float3(dot(Vn,Tangent), dot(Vn,Binormal), a);
	s  *= Height/a;
	float2 ds = s.xy;
	
	// current size of search window
	float size = 1.0/linear_search_steps;

	// current depth position
	float depth = 0.0;

	// search front to back for first point inside object
	for( int i = 0; i < linear_search_steps-1; i++ )
	{
		float t = 1-tex2D(reliefmap,TexCoord+ds*depth).r;
		if(depth < t)
			depth += size;
	}

	// recurse around first point (depth) for closest match
	for( int i = 0; i < binary_search_steps; i++ )
	{
		size *= 0.5;
		float t = 1-tex2D(reliefmap,TexCoord+ds*depth).r;
		if(depth < t)
			depth += (2*size);
		
		depth -= size;
	}

	return TexCoord+ds*depth;
}

float4 ps2a_PS( VS_OUT psInStruct ):COLOR
{
    psInStruct.Tex = ReliefMapping(HeightMapSampler,psInStruct.Tex,psInStruct.vPos,psInStruct.Tangent,psInStruct.Binormal,float3(psInStruct.vPos.w,psInStruct.Tangent.w,psInStruct.Binormal.w));
   
    float4 color = tex2D(ColorMapSampler, psInStruct.Tex);					// fetch color map
    float4 bumpNormal = 2 * (tex2D(BumpMapSampler, psInStruct.Tex) - 0.5); // fetch bump map
    float4 gloss = tex2D(GlossMapSampler, psInStruct.Tex) * vecSkill41[0];
    
    //light1
    float3 LightDir1 = normalize(psInStruct.Light1.xyz);
    float3 ViewDir1 = normalize(psInStruct.View);       
    float4 diff1 = saturate(dot(bumpNormal, LightDir1));    // diffuse component 
    float shadow1 = saturate(4 * diff1);
    float3 Reflect1 = normalize(2 * diff1 * bumpNormal - LightDir1);  // R
    float4 spec1 = pow(saturate(dot(Reflect1, ViewDir1)), vecSkill41[1]); 
    float4 Attenuation1 = saturate(dot(psInStruct.Light1.w, psInStruct.Light1.w));
    
    //light2
    float3 LightDir2 = normalize(psInStruct.Light2.xyz);
    float3 ViewDir2 = normalize(psInStruct.View);
    float4 diff2 = saturate(dot(bumpNormal, LightDir2));    // diffuse component
    float shadow2 = saturate(4 * diff2);
    float3 Reflect2 = normalize(2 * diff2 * bumpNormal - LightDir2);  // R
    float4 spec2 = pow(saturate(dot(Reflect2, ViewDir2)), vecSkill41[1]);
    float4 Attenuation2 = saturate(dot(psInStruct.Light2.w, psInStruct.Light2.w));
    
    //light3
    float3 LightDir3 = normalize(psInStruct.Light3.xyz);
    float3 ViewDir3 = normalize(psInStruct.View);
    float4 diff3 = saturate(dot(bumpNormal, LightDir3));    // diffuse component
    float shadow3 = saturate(4 * diff3);
    float3 Reflect3 = normalize(2 * diff3 * bumpNormal - LightDir3);  // R
    float4 spec3 = pow(saturate(dot(Reflect3, ViewDir3)), vecSkill41[1]);
    float4 Attenuation3 = saturate(dot(psInStruct.Light3.w, psInStruct.Light3.w));

    return
    (
       color*vecAmbient*vecSkill41[2] + //ambient
	    ((shadow1 * (color * diff1 + (spec1*gloss)) * (1 -Attenuation1))*vecLightColor[0]) +
	    ((shadow2 * (color * diff2 + (spec2*gloss)) * (1 -Attenuation2))*vecLightColor[1]) +
	    ((shadow3 * (color * diff3 + (spec3*gloss)) * (1 -Attenuation3))*vecLightColor[2])
    );
}



//////////////////////////////////////////////
///////////3 lights in two passes/////////////
//////////////////ps 2.0//////////////////////
//////////////////////////////////////////////

//calculates the parallax mapping texcoords
float2 ParallaxMapping(sampler2D heightmap,float3 vEye, float2 vCoord)
{
	vEye = normalize(vEye);

	float fDepth = tex2D(heightmap, vCoord).r;
	float2 vHalfOffset = vEye.xy * (fDepth) * Height;

	fDepth = (fDepth + tex2D(heightmap, vCoord + vHalfOffset).r) * 0.5;
	vHalfOffset = vEye.xy * (fDepth) * Height;

	fDepth = (fDepth + tex2D(heightmap, vCoord + vHalfOffset).r) * 0.5;
	vHalfOffset = vEye.xy * (fDepth) * Height;

	return vCoord + vHalfOffset;
}

//Pass1
VS_OUT ps20_P1_VS(float4 Pos : POSITION, float2 texcoord0 : TEXCOORD0, float3 Normal : NORMAL, float4 Tangent : TEXCOORD2  )
{
    VS_OUT Out = (VS_OUT)0;
    Out.Pos = mul(Pos, matWorldViewProj);	// transform Position

    // compute the 3x3 tranform matrix
    // to transform from world space to tangent space
    float3x3 worldToTangentSpace;
    worldToTangentSpace[0] = mul(Tangent.xyz, matWorld);
    worldToTangentSpace[1] = mul(cross(Tangent.xyz, Normal)*Tangent.w, matWorld);
    worldToTangentSpace[2] = mul(Normal, matWorld);

    Out.Tex = texcoord0;

    float3 PosWorld = mul(Pos, matWorld);

    float3 Viewer = PosWorld - vecViewPos;
    Out.View = mul(worldToTangentSpace, - Viewer);		// V

    //light 1
    float3 Light1 = PosWorld - vecLightPos[0];
    Out.Light1.xyz = mul(worldToTangentSpace, -Light1);	// L
    Out.Light1.w = distance(PosWorld,vecLightPos[0])/vecLightPos[0].w;
    
    //light 2
    float3 Light2 = PosWorld - vecLightPos[1];
    Out.Light2.xyz = mul(worldToTangentSpace, -Light2);	// L
    Out.Light2.w = distance(PosWorld,vecLightPos[1])/vecLightPos[1].w;
    
    float ofog = 1 - (distance(PosWorld, vecViewPos) - vecFog.x) * (vecFog.z);
    Out.Fog = ofog;

   return Out;
}


float4 ps20_P1_PS( VS_OUT psInStruct ):COLOR
{
    psInStruct.Tex = ParallaxMapping(HeightMapSampler,psInStruct.View,psInStruct.Tex);
   
    float4 color = tex2D(ColorMapSampler, psInStruct.Tex);					// fetch color map
    float4 bumpNormal = 2 * (tex2D(BumpMapSampler, psInStruct.Tex) - 0.5); // fetch bump map
    float4 gloss = tex2D(GlossMapSampler, psInStruct.Tex) * vecSkill41[0];
    
    //light1
    float3 LightDir1 = normalize(psInStruct.Light1.xyz);
    float3 ViewDir1 = normalize(psInStruct.View);       
    float4 diff1 = saturate(dot(bumpNormal, LightDir1));    // diffuse component 
    float shadow1 = saturate(4 * diff1);
    float3 Reflect1 = normalize(2 * diff1 * bumpNormal - LightDir1);  // R
    float4 spec1 = pow(saturate(dot(Reflect1, ViewDir1)), vecSkill41[1]); 
    float4 Attenuation1 = saturate(dot(psInStruct.Light1.w, psInStruct.Light1.w));
    
    //light2
    float3 LightDir2 = normalize(psInStruct.Light2.xyz);
    float3 ViewDir2 = normalize(psInStruct.View);
    float4 diff2 = saturate(dot(bumpNormal, LightDir2));    // diffuse component
    float shadow2 = saturate(4 * diff2);
    float3 Reflect2 = normalize(2 * diff2 * bumpNormal - LightDir2);  // R
    float4 spec2 = pow(saturate(dot(Reflect2, ViewDir2)), vecSkill41[1]);
    float4 Attenuation2 = saturate(dot(psInStruct.Light2.w, psInStruct.Light2.w));

    return
    (
       color*vecAmbient*vecSkill41[2] + //ambient
	    ((shadow1 * (color * diff1 + (spec1*gloss)) * (1 -Attenuation1))*vecLightColor[0]) +
	    ((shadow2 * (color * diff2 + (spec2*gloss)) * (1 -Attenuation2))*vecLightColor[1])
    );
}


//Pass2
VS_OUT ps20_P2_VS(float4 Pos : POSITION, float2 texcoord0 : TEXCOORD0, float3 Normal : NORMAL, float4 Tangent : TEXCOORD2  )
{
    VS_OUT Out = (VS_OUT)0;
    Out.Pos = mul(Pos, matWorldViewProj);	// transform Position

    // compute the 3x3 tranform matrix
    // to transform from world space to tangent space
    float3x3 worldToTangentSpace;
    worldToTangentSpace[0] = mul(Tangent.xyz, matWorld);
    worldToTangentSpace[1] = mul(cross(Tangent.xyz, Normal)*Tangent.w, matWorld);
    worldToTangentSpace[2] = mul(Normal, matWorld);

    Out.Tex = texcoord0;

    float3 PosWorld = mul(Pos, matWorld);
	
    float3 Viewer = PosWorld - vecViewPos;
    Out.View = mul(worldToTangentSpace, - Viewer);		// V

    //light 1
    float3 Light3 = PosWorld - vecLightPos[2];
    Out.Light3.xyz = mul(worldToTangentSpace, -Light3);	// L
    Out.Light3.w = distance(PosWorld,vecLightPos[2])/vecLightPos[2].w;

   return Out;
}


float4 ps20_P2_PS( VS_OUT psInStruct ):COLOR
{
    psInStruct.Tex = ParallaxMapping(HeightMapSampler,psInStruct.View,psInStruct.Tex);
   
    float4 color = tex2D(ColorMapSampler, psInStruct.Tex);					// fetch color map
    float4 bumpNormal = 2 * (tex2D(BumpMapSampler, psInStruct.Tex) - 0.5); // fetch bump map
    float4 gloss = tex2D(GlossMapSampler, psInStruct.Tex) * vecSkill41[0];
    
    //light3
    float3 LightDir3 = normalize(psInStruct.Light3.xyz);
    float3 ViewDir3 = normalize(psInStruct.View);       
    float4 diff3 = saturate(dot(bumpNormal, LightDir3));    // diffuse component 
    float shadow3 = saturate(4 * diff3);
    float3 Reflect3 = normalize(2 * diff3 * bumpNormal - LightDir3);  // R
    float4 spec3 = pow(saturate(dot(Reflect3, ViewDir3)), vecSkill41[1]); 
    float4 Attenuation3 = saturate(dot(psInStruct.Light3.w, psInStruct.Light3.w));

    return
    (
       color*vecAmbient*vecSkill41[2] + //ambient
	    ((shadow3 * (color * diff3 + (spec3*gloss)) * (1 -Attenuation3))*vecLightColor[2])
    );
}



//////////////////////////////////////////////
/////////////////techniques///////////////////
//////////////////////////////////////////////

technique NormalMapping_2_a
{
	pass P0
	{
		alphablendenable = false;
		srcblend = zero;

		// compile shaders
		VertexShader = compile vs_2_0 ps2a_VS();
		PixelShader  = compile ps_2_a ps2a_PS();
	}
}

technique NormalMapping_2_0
{
	pass P0
	{
		alphablendenable = false;
		srcblend = zero;

		// compile shaders
		VertexShader = compile vs_2_0 ps20_P1_VS();
		PixelShader  = compile ps_2_0 ps20_P1_PS();
	}
	
	pass P1
	{
		alphablendenable = true;
		srcblend = one;
		destblend = one;

		// compile shaders
		VertexShader = compile vs_2_0 ps20_P2_VS();
		PixelShader  = compile ps_2_0 ps20_P2_PS();
	}
}

technique BumpMapping_FFP
{
	pass P0
	{
		// set texture stage states
		Texture[0] = <entSkin2>;
		Texture[1] = <entSkin1>;
		TextureFactor = <vecLightColor[0]>;

		ColorArg1[0] = Texture;
		ColorOp[0] = DotProduct3;
		ColorArg2[0] = Diffuse;

		ColorArg1[1] = Texture;
		ColorOp[1] = Modulate;
		ColorArg2[1] = Current;

		ColorArg1[2] = Current;
		ColorOp[2] = Modulate;
		ColorArg2[2] = TFactor;
	}
}

technique Empty{ pass P0{} }